==> japhar-0.04.tar.gz: japhar-0.04/lib/libnative/java.lang/system.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   system.c -- native methods for java/lang/System.
   Created: Chris Toshok <toshok@hungry.com>, 28-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jni.h"
#include "log.h"
#include "objects.h"
#include "array-class.h"
#include "ClazzFile.h"
#include "interp.h"
#include "jniint.h"

#include <assert.h>
#include <sys/time.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#if HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif
#if defined( FREEBSD )
#include <sys/param.h> /* for MAXPATHLEN */
#elif IRIX
#include <nl_types.h>
#define MAXPATHLEN NL_MAXPATHLEN
#elif HURD
#include <limits.h>
#define MAXPATHLEN _POSIX_PATH_MAX
#else
#include <limits.h>
#define MAXPATHLEN PATH_MAX
#endif

#define MYLOG "Native"

JNIEXPORT jlong JNICALL
Java_java_lang_System_currentTimeMillis(JNIEnv *env,
					jclass cls)
{
  jlong result;
  struct timeval tp;
  
  if (gettimeofday(&tp, NULL) == -1)
    (*env)->FatalError(env, "gettimeofday call failed.");
  
  result = (jlong)tp.tv_sec;
  result *= 1000;
  result += (tp.tv_usec / 1000);

  return result;
}

JNIEXPORT void JNICALL
Java_java_lang_System_arraycopy(JNIEnv *env,
				jclass cls,
				jobject src,
				jint src_position,
				jobject dest,
				jint dest_position,
				jint length)
{
  jint el_size;

  /* XXX lots of checking and exception throwing needs to happen here. */

  el_size = INSTANCE_DATA(src)->fields[ARRAY_ELSIZE_INDEX].value.i;

  memcpy((char*)INSTANCE_DATA(dest)->fields[ARRAY_BODY_INDEX].value.l + dest_position * el_size,
         (char*)INSTANCE_DATA(src)->fields[ARRAY_BODY_INDEX].value.l + src_position * el_size,
         length * el_size);
}

JNIEXPORT jint JNICALL
Java_java_lang_System_identityHashCode(JNIEnv *env,
				       jclass cls,
				       jobject x)
{
  if (x == NULL) return 0;
  else
    {
      jclass object_class = (*env)->FindClass(env, "java/lang/Object");
	  
      jmethodID hashcode = (*env)->GetMethodID(env, object_class, "hashCode", "()I");
	  
      return (*env)->CallNonvirtualIntMethod(env, x, hashcode);
    }
}

/**
 * System properties. The following properties are guaranteed to be defined:
 * <dl>
 * <dt>java.version			<dd>Java version number
 * <dt>java.vendor			<dd>Java vendor specific string
 * <dt>java.vendor.url		<dd>Java vendor URL
 * <dt>java.home			<dd>Java installation directory
 * <dt>java.class.version	<dd>Java class version number
 * <dt>java.class.path		<dd>Java classpath
 * <dt>os.name				<dd>Operating System Name
 * <dt>os.arch				<dd>Operating System Architecture
 * <dt>os.version			<dd>Operating System Version
 * <dt>file.separator		<dd>File separator ("/" on Unix)
 * <dt>path.separator		<dd>Path separator (":" on Unix)
 * <dt>line.separator		<dd>Line separator ("\n" on Unix)
 * <dt>user.name			<dd>User account name
 * <dt>user.home			<dd>User home directory
 * <dt>user.dir				<dd>User's current working directory
 * </dl>
 */

static propertySpec defaults[] = {
  { "user.language",		"en" },
  { "file.encoding.pkg",	"sun.io" },
  { "file.encoding",		"8859_1" },
  { "java.version", 		"1.1" },
  { "java.vendor",			"The Hungry Programmers" },
  { "java.vendor.url",		"http://www.hungry.com" },
  { "java.home",			INSTALL_DIR },
  { "java.class.version",	"45.3" },
  { "file.separator",		"/" },
  { "path.separator",		":" },
  { "line.separator",		"\n" },
  { "\0" }
};

JNIEXPORT jobject JNICALL
Java_java_lang_System_initProperties(JNIEnv *env,
				     jclass cls,
				     jobject props)
{
  jclass hashtable;
  jmethodID hash_put;
  jstring key, value;
  JavaVM *vm;
  HungryJavaVM *jvm;
  char *home, *user;
  char cwd[MAXPATHLEN + 1];
  int i;
#if HAVE_SYS_UTSNAME_H
  struct utsname utsname;
#endif

  JAVARLOG0(MYLOG, 1, "in java_lang_System_initProperties()\n");

  hashtable = (*env)->FindClass(env, "java/util/Hashtable");
  if ((*env)->ExceptionOccurred(env)) return NULL;

  hash_put = (*env)->GetMethodID(env, hashtable, "put", "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;");

  i = 0;

  while (defaults[ i ].key[0] != '\0')
    {
      JAVARLOG2(MYLOG, 2, "  putting property  key(%s), value(%s)\n", 
		defaults[i].key, defaults[i].value);

      key = (*env)->NewStringUTF(env, defaults[i].key);
      value = (*env)->NewStringUTF(env, defaults[i].value);

      /* these need to be global or they'll be collected when we return. */
      key = (*env)->NewGlobalRef(env, key);
      value = (*env)->NewGlobalRef(env, value);

      (*env)->CallVoidMethod(env, props, hash_put, key, value);

      i++;
    }

  /* Add the java.class.path */
  key = (*env)->NewStringUTF(env, "java.class.path");
  (*env)->GetJavaVM(env, &vm);
  jvm = (HungryJavaVM*)vm;
  value = (*env)->NewStringUTF(env, jvm->_args.classpath);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the user's name. */
  key = (*env)->NewStringUTF(env, "user.name");
  user = getenv("LOGNAME");
  if (user == NULL)
    user = "";
  value = (*env)->NewStringUTF(env, user);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the user's home directory. */
  key = (*env)->NewStringUTF(env, "user.home");
  home = getenv("HOME");
  if (home == NULL)
    home = "";
  value = (*env)->NewStringUTF(env, home);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);
  
  /* add the present working directory. */
  key = (*env)->NewStringUTF(env, "user.dir");
  getcwd(cwd, MAXPATHLEN);
  value = (*env)->NewStringUTF(env, cwd);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

#if HAVE_SYS_UTSNAME_H
  uname(&utsname);
#endif

  /* add the OS name and version*/
  key = (*env)->NewStringUTF(env, "os.name");
#if HAVE_SYS_UTSNAME_H
  value = (*env)->NewStringUTF(env, utsname.sysname);
#else
  value = (*env)->NewStringUTF(env, OS_NAME);
#endif

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the OS name and version*/
  key = (*env)->NewStringUTF(env, "os.version");
#if HAVE_SYS_UTSNAME_H
  value = (*env)->NewStringUTF(env, utsname.release);
#else
  value = (*env)->NewStringUTF(env, OS_REV);
#endif

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  return props;
}

==> japhar-0.05.tar.gz: japhar-0.05/lib/libnative/java.lang/system.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   system.c -- native methods for java/lang/System.
   Created: Chris Toshok <toshok@hungry.com>, 28-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jni.h"
#include "log.h"
#include "objects.h"
#include "array-class.h"
#include "ClazzFile.h"
#include "interp.h"
#include "jniint.h"

#include <assert.h>
#include <sys/time.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#if HAVE_SYS_UTSNAME_H
#include <sys/utsname.h>
#endif
#if defined( FREEBSD )
#include <sys/param.h> /* for MAXPATHLEN */
#elif IRIX
#include <nl_types.h>
#define MAXPATHLEN NL_MAXPATHLEN
#elif HURD
#include <limits.h>
#define MAXPATHLEN _POSIX_PATH_MAX
#else
#include <limits.h>
#define MAXPATHLEN PATH_MAX
#endif

#define MYLOG "Native"

JNIEXPORT jlong JNICALL
Java_java_lang_System_currentTimeMillis(JNIEnv *env,
					jclass cls)
{
  jlong result;
  struct timeval tp;
  
  if (gettimeofday(&tp, NULL) == -1)
    (*env)->FatalError(env, "gettimeofday call failed.");
  
  result = (jlong)tp.tv_sec;
  result *= 1000;
  result += (tp.tv_usec / 1000);

  return result;
}

JNIEXPORT void JNICALL
Java_java_lang_System_arraycopy(JNIEnv *env,
				jclass cls,
				jobject src,
				jint src_position,
				jobject dest,
				jint dest_position,
				jint length)
{
  jint el_size;
  ClazzFile *dest_cls, *src_cls;
  FieldStruct *dest_body_field;
  FieldStruct *src_body_field;
  FieldStruct *el_size_field;
  jvalue val;
  char *dest_body, *src_body;

  dest_cls = (*env)->GetObjectClass(env, dest);
  src_cls = (*env)->GetObjectClass(env, src);

  dest_body_field = dest_cls->fields[ARRAY_BODY_INDEX];
  src_body_field = src_cls->fields[ARRAY_BODY_INDEX];

  el_size_field = src_cls->fields[ARRAY_ELSIZE_INDEX];

  /* XXX lots of checking and exception throwing needs to happen here. */

  get_instance_field(src, el_size_field, &val);
  el_size = val.i;

  get_instance_field(dest, dest_body_field, &val);
  dest_body = (char*)val.l;

  get_instance_field(src, src_body_field, &val);
  src_body = (char*)val.l;

  memcpy(dest_body + dest_position * el_size,
         src_body + src_position * el_size,
         length * el_size);
}

JNIEXPORT jint JNICALL
Java_java_lang_System_identityHashCode(JNIEnv *env,
				       jclass cls,
				       jobject x)
{
  if (x == NULL) return 0;
  else
    {
      jclass object_class = (*env)->FindClass(env, "java/lang/Object");
	  
      jmethodID hashcode = (*env)->GetMethodID(env, object_class, "hashCode", "()I");
	  
      return (*env)->CallNonvirtualIntMethod(env, x, hashcode);
    }
}

/**
 * System properties. The following properties are guaranteed to be defined:
 * <dl>
 * <dt>java.version			<dd>Java version number
 * <dt>java.vendor			<dd>Java vendor specific string
 * <dt>java.vendor.url		<dd>Java vendor URL
 * <dt>java.home			<dd>Java installation directory
 * <dt>java.class.version	<dd>Java class version number
 * <dt>java.class.path		<dd>Java classpath
 * <dt>os.name				<dd>Operating System Name
 * <dt>os.arch				<dd>Operating System Architecture
 * <dt>os.version			<dd>Operating System Version
 * <dt>file.separator		<dd>File separator ("/" on Unix)
 * <dt>path.separator		<dd>Path separator (":" on Unix)
 * <dt>line.separator		<dd>Line separator ("\n" on Unix)
 * <dt>user.name			<dd>User account name
 * <dt>user.home			<dd>User home directory
 * <dt>user.dir				<dd>User's current working directory
 * </dl>
 * It will also define the java.compiler if env(JAVA_COMPILER) is defined
 */

static propertySpec defaults[] = {
  { "user.language",		"en" },
  { "file.encoding.pkg",	"sun.io" },
  { "file.encoding",		"8859_1" },
  { "java.version", 		"1.1" },
  { "java.vendor",			"The Hungry Programmers" },
  { "java.vendor.url",		"http://www.hungry.com" },
  { "java.home",			INSTALL_DIR },
  { "java.class.version",	"45.3" },
  { "file.separator",		"/" },
  { "path.separator",		":" },
  { "line.separator",		"\n" },
  { "\0" }
};

JNIEXPORT jobject JNICALL
Java_java_lang_System_initProperties(JNIEnv *env,
				     jclass cls,
				     jobject props)
{
  jclass hashtable;
  jmethodID hash_put;
  jstring key, value;
  JavaVM *vm;
  HungryJavaVM *jvm;
  char *home, *user, *jit;
  char cwd[MAXPATHLEN + 1];
  int i;
#if HAVE_SYS_UTSNAME_H
  struct utsname utsname;
#endif

  JAVARLOG0(MYLOG, 1, "in java_lang_System_initProperties()\n");

  hashtable = (*env)->FindClass(env, "java/util/Hashtable");
  if ((*env)->ExceptionOccurred(env)) return NULL;

  hash_put = (*env)->GetMethodID(env, hashtable, "put", "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;");

  i = 0;

  while (defaults[ i ].key[0] != '\0')
    {
      JAVARLOG2(MYLOG, 2, "  putting property  key(%s), value(%s)\n", 
		defaults[i].key, defaults[i].value);

      key = (*env)->NewStringUTF(env, defaults[i].key);
      value = (*env)->NewStringUTF(env, defaults[i].value);

      /* these need to be global or they'll be collected when we return. */
      key = (*env)->NewGlobalRef(env, key);
      value = (*env)->NewGlobalRef(env, value);

      (*env)->CallVoidMethod(env, props, hash_put, key, value);

      i++;
    }

  /* Add the java.class.path */
  key = (*env)->NewStringUTF(env, "java.class.path");
  (*env)->GetJavaVM(env, &vm);
  jvm = (HungryJavaVM*)vm;
  value = (*env)->NewStringUTF(env, jvm->_args.classpath);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the user's name. */
  key = (*env)->NewStringUTF(env, "user.name");
  user = getenv("LOGNAME");
  if (user == NULL)
    user = "";
  value = (*env)->NewStringUTF(env, user);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the user's home directory. */
  key = (*env)->NewStringUTF(env, "user.home");
  home = getenv("HOME");
  if (home == NULL)
    home = "";
  value = (*env)->NewStringUTF(env, home);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);
  
  /* add the JIT compiler name if env(JAVA_COMPILER) exists. */
  jit = getenv("JAVA_COMPILER");
  if (NULL != jit) {
    key = (*env)->NewStringUTF(env, "java.compiler");
    value = (*env)->NewStringUTF(env, jit);

    /* these need to be global or they'll be collected when we return. */
    key = (*env)->NewGlobalRef(env, key);
    value = (*env)->NewGlobalRef(env, value);

    (*env)->CallVoidMethod(env, props, hash_put, key, value);
  }
  
  /* add the present working directory. */
  key = (*env)->NewStringUTF(env, "user.dir");
  getcwd(cwd, MAXPATHLEN);
  value = (*env)->NewStringUTF(env, cwd);

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

#if HAVE_SYS_UTSNAME_H
  uname(&utsname);
#endif

  /* add the OS name and version*/
  key = (*env)->NewStringUTF(env, "os.name");
#if HAVE_SYS_UTSNAME_H
  value = (*env)->NewStringUTF(env, utsname.sysname);
#else
  value = (*env)->NewStringUTF(env, OS_NAME);
#endif

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  /* add the OS name and version*/
  key = (*env)->NewStringUTF(env, "os.version");
#if HAVE_SYS_UTSNAME_H
  value = (*env)->NewStringUTF(env, utsname.release);
#else
  value = (*env)->NewStringUTF(env, OS_REV);
#endif

  /* these need to be global or they'll be collected when we return. */
  key = (*env)->NewGlobalRef(env, key);
  value = (*env)->NewGlobalRef(env, value);

  (*env)->CallVoidMethod(env, props, hash_put, key, value);

  return props;
}
